const proxyquire = require('proxyquire');
const chaiAsPromised = require('chai-as-promised');

const chai = require('chai');
const { match, stub, resetHistory } = require('sinon');
const { makeMockModels } = require('sequelize-test-helpers');
const { describe } = require('mocha');

const { addonMessages } = require('../../custom/constants');
const fixtures = require('../fixtures');

chai.use(chaiAsPromised);

const expect = chai.expect;

describe('src/services/addon', () => {
    const addon = {
        update: stub(),
        findOne: stub(),
        create: stub(),
        findAll: stub(),
    };

    const mockModels = makeMockModels({ addon });
    const addonService = proxyquire('../../services/addon.service', {
        '../models': mockModels,
    });

    const mockAddon = { ...fixtures.addonResponse, destroy: stub() };

    let result;
    describe('deleteAddon', () => {
        context('Addon does not exist with the provided id', () => {
            before(() => {
                addon.findOne.resolves(null);
            });

            after(resetHistory);

            it('called addon.findOne', async () => {
                await expect(addonService.deleteAddon(fixtures.addonResponse.id)).to.eventually.be.rejectedWith(Error, addonMessages.noIdFound);
            });
            it("didn't call addon.destroy if addon is not avaliable", () => {
                expect(mockAddon.destroy).not.to.be.called;
            });
        });
        context('Addon exist with the provided id', () => {
            before(async () => {
                mockAddon.destroy.resolves(mockAddon);
                addon.findOne.resolves(mockAddon);
                result = await addonService.deleteAddon(fixtures.addonResponse.id);
            });

            after(resetHistory);

            it('called addon.findOne', async () => {
                expect(addon.findOne).to.have.been.calledWith(match({ where: { id: fixtures.addonResponse.id } }));
            });

            it('called addon.destroy', () => {
                expect(mockAddon.destroy).to.have.been.called;
            });

            it('deleted the addon and return true', () => {
                expect(result).to.deep.equal(true);
             });

        });
    });

    describe('Create Addon', () => {
        context('Addon already exist with provided id', () => {
            it('Does not create an addon on error', async () => {
                const payload = fixtures.addonRequest;
                addon.create.rejects({SequelizeUniqueConstraintError: addonMessages.alreadyExists});
                await expect(addonService.createAddon(payload)).to.eventually.be.rejectedWith(Error, addonMessages.creationFailed);
            });
        });
        context('Addon created successfully', () => {
            it('Does create an addon on valid data', async () => {
                const payload = fixtures.addonRequest;
                addon.create.resolves(fixtures.addonResponse);
                await expect(addonService.createAddon(payload)).to.eventually.be.equals(fixtures.addonResponse);
            });
        });
    });

    describe('Get Addon', () => {
        context('Addon does not exist with provided id', () => {
            it('Does not get an addon on error', async () => {
                const id = fixtures.addonRequest.id;
                addon.findOne.resolves(null);
                await expect(addonService.getAddon(id)).to.eventually.be.rejectedWith(Error, addonMessages.notFound);
            });
        });
        context('Addon get successfully', () => {
            it('Does get an addon on valid id', async () => {
                const id = fixtures.addonRequest.id;
                addon.findOne.resolves(fixtures.addonResponse);
                await expect(addonService.getAddon(id)).to.eventually.be.equals(fixtures.addonResponse);
            });
        });
    });
    describe('List Addon', () => {
        context('No addon avaliable in model', () => {
            it('Does not get any addon list', async () => {
                addon.findAll.resolves(null);
                await expect(addonService.listAddon()).to.eventually.be.rejectedWith(Error, addonMessages.notFound);
            });
        });
        context('Get an addon list successfully', () => {
            it('display a list of addon', async () => {
                addon.findAll.resolves(fixtures.addonResponse);
                await expect(addonService.listAddon()).to.eventually.be.equal(fixtures.addonResponse);
            });
        });
    });

    describe('Update Addon', () => {
        context('Addon does not exist with provided id', () => {
            it('Does not update an addon on error', async () => {
                const payload = {...fixtures.addonRequest, id: 1};
                addon.update.resolves(null);
                await expect(addonService.updateAddon(payload)).to.eventually.be.rejectedWith(Error, addonMessages.updateFailed);
            });
        });
        context('Addon updated successfully', () => {
            it('Does update an addon on valid data', async () => {
                const payload = {...fixtures.addonRequest, id: 1};
                addon.update.resolves(fixtures.addonResponse);
                await expect(addonService.updateAddon(payload)).to.eventually.be.equals(true);
            });
        });
    });
});
