const proxyquire = require('proxyquire');
const chaiAsPromised = require('chai-as-promised');

const chai = require('chai');
const { match, stub, resetHistory } = require('sinon');
const { makeMockModels } = require('sequelize-test-helpers');
const { describe } = require('mocha');

const { addonAddonItemMessages } = require('../../custom/constants');
const fixtures = require('../fixtures');

chai.use(chaiAsPromised);

const expect = chai.expect;

describe('src/services/addonAddonItem', () => {
    const addon_addon_item = {
        update: stub(),
        findOne: stub(),
        create: stub(),
        findAll: stub()
    };

    const mockModels = makeMockModels({ addon_addon_item });
    const addonAddonItemService = proxyquire('../../services/addonAddonItem.service', {
        '../models': mockModels,
    });

    const mockAddonAddonItem = { ...fixtures.addonAddonItem, id: fixtures.id,  destroy: stub() };

    let result;
    describe('deleteAddonAddonItem', () => {
        context('AddomAddonItem does not exist with the provided id', () => {
            before(() => {
                addon_addon_item.findOne.resolves(null);
            });

            after(resetHistory);

            it('called addon.findOne', async () => {
                await expect(addonAddonItemService.deleteAddonAddonItem(fixtures.id)).to.eventually.be.rejectedWith(Error, addonAddonItemMessages.deleteFailed);
            });
            it("didn't call addonAddonItem.destroy if addonAddonItem is not avaliable", () => {
                expect(mockAddonAddonItem.destroy).not.to.be.called;
            });
        });
        context('addonAddonItem exist with the provided id', () => {
            before(async () => {
                mockAddonAddonItem.destroy.resolves(mockAddonAddonItem);
                addon_addon_item.findOne.resolves(mockAddonAddonItem);
                result = await addonAddonItemService.deleteAddonAddonItem(fixtures.id);
            });

            after(resetHistory);

            it('called addonAddonItem.findOne', async () => {
                expect(addon_addon_item.findOne).to.have.been.calledWith(match({ where: { id: fixtures.id } }));
            });

            it('called addon.destroy', () => {
                expect(mockAddonAddonItem.destroy).to.have.been.called;
            });

            it('deleted the addon and return true', () => {
                expect(result).to.deep.equal(true);
            });

        });
    });

    describe('Create AddonAddonItem', () => {
        context('AddonAddonItem already exist with provided id', () => {
            it('Does not create an addonAddonItem on error', async () => {
                const payload = fixtures.addonAddonItem;
                addon_addon_item.create.rejects({ SequelizeUniqueConstraintError: addonAddonItemMessages.creationFailed });
                await expect(addonAddonItemService.createAddonAddonItem(payload)).to.eventually.be.rejectedWith(Error, addonAddonItemMessages.creationFailed);
            });
        });
        context('AddonAddonItem created successfully', () => {
            it('Does create an addonAddonItem on valid data', async () => {
                const payload = fixtures.addonAddonItem;
                const response = { ...fixtures.addonAddonItem, id: fixtures.id };
                addon_addon_item.create.resolves(response);
                await expect(addonAddonItemService.createAddonAddonItem(payload)).to.eventually.be.equals(response);
            });
        });
    });

    describe('Get AddonAddonItem', () => {
        context('Addon does not exist with provided id', () => {
            it('Does not get an addonAddonItem on error', async () => {
                const id = fixtures.id;
                addon_addon_item.findOne.resolves(null);
                await expect(addonAddonItemService.getAddonAddonItem(id)).to.eventually.be.rejectedWith(Error, addonAddonItemMessages.notFound);
            });
        });
        context('Addon get successfully', () => {
            it('Does get an addon on valid id', async () => {
                const id = fixtures.id;
                addon_addon_item.findOne.resolves(fixtures.addonAddonItem);
                await expect(addonAddonItemService.getAddonAddonItem(id)).to.eventually.be.equals(fixtures.addonAddonItem);
            });
        });
    });
    describe('List AddonAddonItem', () => {
        context('No addonAddonItem avaliable in model', () => {
            it('Does not get any addonAddonItem list', async () => {
                addon_addon_item.findAll.resolves(null);
                await expect(addonAddonItemService.listAddonAddonItem()).to.eventually.be.rejectedWith(Error, addonAddonItemMessages.notFound);
            });
        });
        context('Get an addonAddonItem list successfully', () => {
            it('display a list of addonAddonItem', async () => {
                addon_addon_item.findAll.resolves(fixtures.addonAddonItem);
                await expect(addonAddonItemService.listAddonAddonItem()).to.eventually.be.equal(fixtures.addonAddonItem);
            });
        });
    });

    describe('Update AddonAddonItem', () => {
        context('AddonAddonItem does not exist with provided id', () => {
            it('Does not update an AddonAddonItem on error', async () => {
                const payload = {...fixtures.addonAddonItem, id: fixtures.id};
                addon_addon_item.update.resolves(null);
                await expect(addonAddonItemService.updateAddonAddonItem(payload)).to.eventually.be.rejectedWith(Error, addonAddonItemMessages.updateFailed);
            });
        });
        context('AddonAddonItem updated successfully', () => {
            it('Does update an AddonAddonItem on valid data', async () => {
                const payload = {...fixtures.addonAddonItem, id: fixtures.id};
                addon_addon_item.update.resolves(fixtures.addonAddonItem);
                await expect(addonAddonItemService.updateAddonAddonItem(payload)).to.eventually.be.equals(true);
            });
        });
    });
});
