const proxyquire = require('proxyquire');
const chaiAsPromised = require('chai-as-promised');

const chai = require('chai');
const { match, stub, resetHistory } = require('sinon');
const { makeMockModels } = require('sequelize-test-helpers');
const { describe } = require('mocha');

const { addonItemMessages } = require('../../custom/constants');
const fixtures = require('../fixtures');

chai.use(chaiAsPromised);

const expect = chai.expect;

describe('src/services/addonItem', () => {
    const addon_item = {
        update: stub(),
        findOne: stub(),
        create: stub(),
        findAll: stub()
    };

    const mockModels = makeMockModels({ addon_item });
    const addonItemService = proxyquire('../../services/addonItem.service', {
        '../models': mockModels,
    });

    const mockAddonItem = { ...fixtures.addonItem, id: fixtures.id,  destroy: stub() };

    let result;
    describe('deleteAddonItem', () => {
        context('AddonItem does not exist with the provided id', () => {
            before(() => {
                addon_item.findOne.resolves(null);
            });

            after(resetHistory);

            it('called addonItem.findOne', async () => {
                await expect(addonItemService.deleteAddonItem(fixtures.id)).to.eventually.be.rejectedWith(Error, addonItemMessages.deleteFailed);
            });
            it("didn't call addonItem.destroy if addonItem is not avaliable", () => {
                expect(mockAddonItem.destroy).not.to.be.called;
            });
        });
        context('addonItem exist with the provided id', () => {
            before(async () => {
                mockAddonItem.destroy.resolves(mockAddonItem);
                addon_item.findOne.resolves(mockAddonItem);
                result = await addonItemService.deleteAddonItem(fixtures.id);
            });

            after(resetHistory);

            it('called addonAddonItem.findOne', async () => {
                expect(addon_item.findOne).to.have.been.calledWith(match({ where: { id: fixtures.id } }));
            });

            it('called addonItem.destroy', () => {
                expect(mockAddonItem.destroy).to.have.been.called;
            });

            it('deleted the addon and return true', () => {
                expect(result).to.deep.equal(true);
            });

        });
    });

    describe('Create AddonItem', () => {
        context('AddonItem already exist with provided id', () => {
            it('Does not create an addonItem on error', async () => {
                const payload = fixtures.addonItem;
                addon_item.create.rejects({ SequelizeUniqueConstraintError: addonItemMessages.creationFailed });
                await expect(addonItemService.createAddonItem(payload)).to.eventually.be.rejectedWith(Error, addonItemMessages.creationFailed);
            });
        });
        context('AddonItem created successfully', () => {
            it('Does create an addonItem on valid data', async () => {
                const payload = fixtures.addonItem;
                const response = { ...fixtures.addonItem, id: fixtures.id };
                addon_item.create.resolves(response);
                await expect(addonItemService.createAddonItem(payload)).to.eventually.be.equals(response);
            });
        });
    });

    describe('Get AddonItem', () => {
        context('AddonItem does not exist with provided id', () => {
            it('Does not get an addonItem on error', async () => {
                const id = fixtures.id;
                addon_item.findOne.resolves(null);
                await expect(addonItemService.getAddonItem(id)).to.eventually.be.rejectedWith(Error, addonItemMessages.notFound);
            });
        });
        context('AddonItem get successfully', () => {
            it('Does get an addonItem on valid id', async () => {
                const id = fixtures.id;
                addon_item.findOne.resolves(fixtures.addonItem);
                await expect(addonItemService.getAddonItem(id)).to.eventually.be.equals(fixtures.addonItem);
            });
        });
    });
    describe('List AddonItem', () => {
        context('No addonItem avaliable in model', () => {
            it('Does not get any addonItem list', async () => {
                addon_item.findAll.resolves(null);
                await expect(addonItemService.listAddonItem()).to.eventually.be.rejectedWith(Error, addonItemMessages.notFound);
            });
        });
        context('Get an addonItem list successfully', () => {
            it('display a list of addonItem', async () => {
                addon_item.findAll.resolves(fixtures.addonItem);
                await expect(addonItemService.listAddonItem()).to.eventually.be.equal(fixtures.addonItem);
            });
        });
    });

    describe('Update AddonItem', () => {
        context('AddonItem does not exist with provided id', () => {
            it('Does not update an AddonItem on error', async () => {
                const payload = {...fixtures.addonItem, id: fixtures.id};
                addon_item.update.resolves(null);
                await expect(addonItemService.updateAddonItem(payload)).to.eventually.be.rejectedWith(Error, addonItemMessages.updateFailed);
            });
        });
        context('AddonItem updated successfully', () => {
            it('Does update an AddonItem on valid data', async () => {
                const payload = {...fixtures.addonAddonItem, id: fixtures.id};
                addon_item.update.resolves(fixtures.addonItem);
                await expect(addonItemService.updateAddonItem(payload)).to.eventually.be.equals(true);
            });
        });
    });
});
